#!/bin/sh

# description: this script downloads the bioassay screen data (.csv) and all compounds associated (.sdf)
#              First the csv file with biological activities is downl
#
# @author Mariusz Butkiewicz
# @date May 2012

num_args=`echo $* | wc -w`

usage()
{
    echo "Usage: pubchem_aid_download.sh [pubchem aid number]"
    echo "param: [pubchem aid number] pubchem aid number reffering to the pubchem bioassay id ( eg. 891, 893 or 1445)"
    echo "description: this script downloads the bioassay screen data (.csv) and all compounds associated (.sdf)"
}

# check whether option -h was set
if [[ "$1" == "-h" ]];
then
    usage
    echo "" 
    echo "Options:"
    echo "  -h       show this help message and exit"
    exit 1
fi

# check numer of cmd parameters given
if [ $num_args -ne 1 ];
then
    usage
    echo ""
    echo "pubchem_aid_download.sh: error: Please provide just one parameter!"
    exit 1
fi

# check if cmd parameter is not numeric
if [[ ! -z $(echo $1 | sed 's/[0-9]//g') ]]
then
     usage
     echo ""
     echo "pubchem_aid_download.sh: error: Please provide a numeric parameter!"
     exit 1
fi

######## xml queries to access PUBCHEM PUG CGI interface ################

poll="<PCT-Data>
  <PCT-Data_input>
    <PCT-InputData>
      <PCT-InputData_request>
        <PCT-Request>
          <PCT-Request_reqid>XXXX</PCT-Request_reqid>
          <PCT-Request_type value=\"status\"/>
        </PCT-Request>
      </PCT-InputData_request>
    </PCT-InputData>
  </PCT-Data_input>
</PCT-Data>"

query_header="<PCT-Data>
  <PCT-Data_input>
    <PCT-InputData>
      <PCT-InputData_download>
        <PCT-Download>
          <PCT-Download_uids>
            <PCT-QueryUids>
              <PCT-QueryUids_ids>
                <PCT-ID-List>
                  <PCT-ID-List_db>pccompound</PCT-ID-List_db>
                  <PCT-ID-List_uids>
"

query_footer="    </PCT-ID-List_uids>
                </PCT-ID-List>
              </PCT-QueryUids_ids>
            </PCT-QueryUids>
          </PCT-Download_uids>
          <PCT-Download_format value=\"sdf\"/>
          <PCT-Download_compression value=\"gzip\"/>
        </PCT-Download>
      </PCT-InputData_download>
    </PCT-InputData>
  </PCT-Data_input>
</PCT-Data>"



query_aid="<PCT-Data>
  <PCT-Data_input>
    <PCT-InputData>
      <PCT-InputData_query>
        <PCT-Query>
          <PCT-Query_type>
            <PCT-QueryType>
              <PCT-QueryType_bas>
                <PCT-QueryAssayData>
                  <PCT-QueryAssayData_output value=\"csv\">4</PCT-QueryAssayData_output>
                  <PCT-QueryAssayData_aids>
                    <PCT-QueryUids>
                      <PCT-QueryUids_ids>
                        <PCT-ID-List>
                          <PCT-ID-List_db>pcassay</PCT-ID-List_db>
                          <PCT-ID-List_uids>
                            <PCT-ID-List_uids_E>$1</PCT-ID-List_uids_E>
                          </PCT-ID-List_uids>
                        </PCT-ID-List>
                      </PCT-QueryUids_ids>
                    </PCT-QueryUids>
                  </PCT-QueryAssayData_aids>
                  <PCT-QueryAssayData_dataset value=\"complete\">0</PCT-QueryAssayData_dataset>
                  <PCT-QueryAssayData_focus>
                    <PCT-Assay-FocusOption>
                      <PCT-Assay-FocusOption_group-results-by value=\"substance\">4</PCT-Assay-FocusOption_group-results-by>
                    </PCT-Assay-FocusOption>
                  </PCT-QueryAssayData_focus>
                </PCT-QueryAssayData>
              </PCT-QueryType_bas>
            </PCT-QueryType>
          </PCT-Query_type>
        </PCT-Query>
      </PCT-InputData_query>
    </PCT-InputData>
  </PCT-Data_input>
</PCT-Data>"

########################################################################################

date

# if csv file exists and is > 0 in size then query
if [ ! -s $1.csv ]
then

    echo "download AID $1"
    echo $query_aid > $1.query_aid.xml

    # AID get csv file
    curl -d @$1.query_aid.xml "http://pubchem.ncbi.nlm.nih.gov/pug/pug.cgi" > $1.status.xml

    poll_id=`cat $1.status.xml | grep reqid | tr '><' '  ' | awk '{print $2}'`

    while [ `grep -c "ftp" $1.status.xml` -eq 0 ] ;
    do
	    echo "csv - waiting (60 seconds)"
	    sleep 60
	    echo $poll | sed 's/XXXX/'$poll_id'/g' > $1.poll.xml

	    curl -d @$1.poll.xml "http://pubchem.ncbi.nlm.nih.gov/pug/pug.cgi" > $1.status.xml

        echo "status: "`grep reqid $1.status.xml`
        echo "error: "`grep messages_E $1.status.xml`
    done

    # Download aid csv file
    aid_url=`cat $1.status.xml | grep ftp | tr '<>' '  ' | awk '{print $2}'` 
    wget $aid_url
    csv_file=`cat $1.status.xml | grep ftp | tr '></' '   ' | awk '{print $7}'`
    mv $csv_file $1.csv.gz
    gunzip $1.csv.gz

    echo "wget aid $1 csv file .. done"

else 

    echo "download AID $1 .. already done"
fi

#prepare compound id to download aid sdf from pubchem
cat $1.csv | sed 's/,,/,\"\",/g' | tr ',' ' ' | awk '{if(NR > 1 && $3 > 0 && NF > 0){ print "                    <PCT-ID-List_uids_E>"$3"</PCT-ID-List_uids_E>"}}' > $1.sid.txt

min=0
max=`cat $1.sid.txt | wc -l`
begin=0
step_size=50000
counter=1
echo "compounds range $begin, $max, `wc -l $1.sid.txt`"

while [[ $begin -lt $max ]]
do
	begin=$(($begin + $step_size))

	echo "compounds range [ $min, $begin], #enries: `awk '{if( NR > '$min' && NR < '$begin'){print $0}}' $1.sid.txt | wc -l`"

	echo $query_header `awk '{if( NR > '$min' && NR <= '$begin'){print $0}}' $1.sid.txt` $query_footer > $1.compound_request.xml

	min=$begin

	curl -d @$1.compound_request.xml "http://pubchem.ncbi.nlm.nih.gov/pug/pug.cgi" > $1.status-$counter.xml

	counter=$(($counter + 1))
done

iter=1
while [[ $iter -lt $counter ]]
do
	poll_id=`cat $1.status-$iter.xml | grep reqid | tr '><' '  ' | awk '{print $2}'`

	while [ `grep -c "ftp" $1.status-$iter.xml` -eq 0 ] ;
	do
        	echo "sdf - waiting (60 seconds)"
        	sleep 60
		#echo `poll.sh $poll_id` > $1.poll.xml
		echo $poll | sed 's/XXXX/'$poll_id'/g' > $1.poll.xml

      	curl -d @$1.poll.xml "http://pubchem.ncbi.nlm.nih.gov/pug/pug.cgi" > $1.status-$iter.xml

		echo "status: "`grep reqid $1.status-$iter.xml`
       	echo "error: "`grep messages_E $1.status-$iter.xml`
	done

	# Download aid csv file
	sdf_url=`cat $1.status-$iter.xml | grep ftp | tr '<>' '  ' | awk '{print $2}'` 
	wget $sdf_url
	sdf_file=`cat $1.status-$iter.xml | grep ftp | tr '></' '   ' | awk '{print $7}'`
	cat $sdf_file >> $1.sdf.gz
	rm -f $sdf_file

	iter=$(($iter + 1))
done

echo "wget aid $1 sdf file .. done"

rm -f $1.status*.xml $1.header.txt $1.footer.txt $1.compound_request.xml $1.sid.txt $1.query_aid.xml $1.poll.xml 

echo "clean up ..  done"

date
